﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Runtime.ConstrainedExecution;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using IDTechZMQ_Server;

namespace ZMQ_Server_App
{
    public partial class Form1 : Form
    {
        
         void deviceListChanged(List<string> connectedDevices)
        {
            if (InvokeRequired)
            {
                try
                {
                    Invoke((MethodInvoker)delegate
                    {
                        deviceListChanged(connectedDevices);
                    });
                    return;
                }
                catch
                {

                }

            }
            deviceList.DataSource = null;
            deviceList.DataSource = connectedDevices;
        }
         void clientListChanged(List<string> connectedClients)
        {
            if (InvokeRequired)
            {
                try
                {
                    Invoke((MethodInvoker)delegate
                    {
                        clientListChanged(connectedClients);
                    });
                    return;
                }
                catch
                {

                }

            }
            clientList.DataSource = null;
            clientList.DataSource = connectedClients;
        }
         void deviceStateChanged(string ident, string client, int state)
        {
            if (InvokeRequired)
            {
                try
                {
                    Invoke((MethodInvoker)delegate
                    {
                        deviceStateChanged( ident,  client,  state);
                    });
                    return;
                }
                catch
                {

                }

            }
            SetOutputText(ident + ":" + client + ":" + state.ToString());
        }
        delegate void SetTextCallback(string text);

        private void SetOutputText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (tbOutput.InvokeRequired)
            {
                try
                {
                    SetTextCallback d = new SetTextCallback(SetOutputText);
                    Invoke(d, new object[] { text });
                }
                catch
                {

                }

            }
            else
            {
               
                try
                {
                    

                    tbOutput.AppendText(text + "\r\n");
                }
                catch (Exception)
                {
                }

            }
        }

        private void setMessageText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (tbOutput.InvokeRequired)
            {
                try
                {
                    SetTextCallback d = new SetTextCallback(setMessageText);
                    Invoke(d, new object[] { text });
                }
                catch
                {

                }

            }
            else
            {
                TextBox tb = messagebox;

                if (text == null || text.Length == 0)
                {
                    tb.Text = "";
                    return;
                }
                try
                {
                    TextBox temp = new TextBox();
                    temp.Text = text;

                    tb.Text = temp.Text;
                    tb.TabStop = false;
                    tb.Invalidate();
                    tb.Update();
                    tb.Refresh();

                }
                catch (Exception)
                {
                }

            }

            text = "";
        }
        public Form1()
        {
            InitializeComponent();
            RemoteAPI.setDeviceListChangedCallback(deviceListChanged);

            RemoteAPI.setClientListChangedCallback(clientListChanged);

            RemoteAPI.setDeviceStateChangedCallback(deviceStateChanged);

            RemoteAPI.setCommunicationCallback(communicationCallback);

            setMessageText("SERVER IS STOPPED");
            string ver2 = Application.ProductVersion;
            string version = "   SDK: " + IDTechSDK.IDT_Device.SDK_Version() + "  /  App: " + ver2;
            this.Text = this.Text + version;
        }
        int logCount = 0;
        public void communicationCallback(bool incoming, string json)
        {
            if (!enableLog.Checked) return;
            if (InvokeRequired)
            {
                try
                {
                    Invoke((MethodInvoker)delegate
                    {
                        communicationCallback(incoming,json);
                    });
                    return;
                }
                catch
                {

                }

            }
            
            string direction = "OUT: >>>>>>>>>>>>>>>>>>>>\r\n";
            if (incoming) direction = "IN: <<<<<<<<<<<<<<<<<<<<\r\n";
            string display = direction + json + "\r\n";
            commCallback.AppendText(display);
            logCount += display.Length;
            if (logCount > 10000)
            {
                commCallback.Text = "";
                logCount = 0;
            }
            

        }

        public List<string> GetLocalIPv4(NetworkInterfaceType _type)
        {
            List<string> output = new List<string>();
            foreach (NetworkInterface item in NetworkInterface.GetAllNetworkInterfaces())
            {
                if (item.NetworkInterfaceType == _type && item.OperationalStatus == OperationalStatus.Up)
                {
                    foreach (UnicastIPAddressInformation ip in item.GetIPProperties().UnicastAddresses)
                    {
                        if (ip.Address.AddressFamily == AddressFamily.InterNetwork)
                        {
                            output.Add(ip.Address.ToString());
                        }
                    }
                }
            }
            return output;
        }
        public List<string> GetAllIPv4()
        {
            List<string> output = new List<string>();
            foreach (NetworkInterface item in NetworkInterface.GetAllNetworkInterfaces())
            {
                if ((item.NetworkInterfaceType == NetworkInterfaceType.Ethernet || item.NetworkInterfaceType == NetworkInterfaceType.Wireless80211) && item.OperationalStatus == OperationalStatus.Up)
                {
                    foreach (UnicastIPAddressInformation ip in item.GetIPProperties().UnicastAddresses)
                    {
                        if (ip.Address.AddressFamily == AddressFamily.InterNetwork)
                        {
                            output.Add(ip.Address.ToString());
                        }
                    }
                }
            }
            output.Add("127.0.0.1");
            return output;
        }

        private void startServer_Click(object sender, EventArgs e)
        {
            

            //try ethernet first
           List<string> ipaddr1 = GetAllIPv4();
            StringBuilder sb = new StringBuilder();

            sb.AppendLine("Please enter IP address to publish server on.");
            foreach(string str in ipaddr1)
            {
                sb.AppendLine(str);
            }
            
            string ipaddr = Microsoft.VisualBasic.Interaction.InputBox(sb.ToString(), "IP Address", ipaddr1[0], -1, -1);
            if (ipaddr.Length == 0)
            {
                setMessageText("NO VALID IP ADDRESS SELECTED");
                return;
            }

            setMessageText("STARTING SERVER. PLEASE WAIT...");

            if (RemoteAPI.SharedController.ZMQServer(true, "ZMQ_SERVER", ipaddr)){
                setMessageText("\"ZMQ SERVER\" STARTED ON " + ipaddr);
                startServer.Enabled = false;
                stopServer.Enabled = true;
            }
            else
            {
                setMessageText("ZMQ SERVER FAILED TO START");
            }

        }

        private void stopServer_Click(object sender, EventArgs e)
        {
            setMessageText("SHUTTING DOWN SERVER. PLEASE WAIT...");
            if (RemoteAPI.SharedController.ZMQServer(false))
            {
                setMessageText("SERVER IS STOPPED");
                startServer.Enabled = true;
                stopServer.Enabled = false;
            }
            else
            {
                setMessageText("ZMQ SERVER FAILED TO STOP");
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            commCallback.Text = "";
            tbOutput.Text = "";
            logCount = 0;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            stopServer_Click(null, null);
        }

        IDTechSDK.Logging.LogManager log = null;
        private void enableLog_CheckedChanged(object sender, EventArgs e)
        {
            IDTechSDK.Logging.LoggingConfiguration config = new IDTechSDK.Logging.LoggingConfiguration();
            config.useCommonLogging = enableLog.Checked;
            config.useDeviceCommunicationLogging = enableLog.Checked;
            config.useZeroMQLogging = enableLog.Checked;
            if (log == null) log = new IDTechSDK.Logging.LogManager();
            log.SetLoggerConfiguration(config);
        }
    }
}
